/*
 * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS HEADER.
 *
 * The contents of this file are subject to the terms of either the GNU
 * General Public License Version 2 only ("GPL") or the Common
 * Development and Distribution License("CDDL") (collectively, the
 * "License"). You may not use this file except in compliance with the
 * License. You can obtain a copy of the License at
 * http://www.netbeans.org/cddl-gplv2.html
 * or nbbuild/licenses/CDDL-GPL-2-CP. See the License for the
 * specific language governing permissions and limitations under the
 * License.  When distributing the software, include this License Header
 * Notice in each file and include the License file at
 * nbbuild/licenses/CDDL-GPL-2-CP.  Sun designates this
 * particular file as subject to the "Classpath" exception as provided
 * by Sun in the GPL Version 2 section of the License file that
 * accompanied this code. If applicable, add the following below the
 * License Header, with the fields enclosed by brackets [] replaced by
 * your own identifying information:
 * "Portions Copyrighted [year] [name of copyright owner]"
 *
 * Contributor(s):
 *
 * Portions Copyrighted 2009 Jaroslav Tulach
 */

package cz.xelfi.quoridor.webidor;

import com.sun.jersey.api.client.GenericType;
import java.util.List;
import com.sun.jersey.api.client.UniformInterfaceException;
import com.sun.jersey.test.framework.JerseyTest;
import java.io.File;
import java.io.FileOutputStream;
import java.io.IOException;
import java.util.Properties;
import javax.ws.rs.core.MediaType;
import org.junit.Test;
import static org.junit.Assert.*;

/**
 *
 * @author Jaroslav Tulach <jtulach@netbeans.org>
 */
public class AllGamesTest extends JerseyTest {
    static {
        System.setProperty("JERSEY_HTTP_PORT", "45420");
    }

    private File dir;

    public AllGamesTest() throws Exception {
        super("cz.xelfi.quoridor.webidor.resources");
    }

    @Override
    public void setUp() throws Exception {
        dir = File.createTempFile("quoridor", ".dir");
        dir.delete();
        System.setProperty("quoridor.dir", dir.getPath());
        dir.mkdirs();
        File passwd = new File(dir, "passwd");
        FileOutputStream os = new FileOutputStream(passwd);
        os.write("Jarda=heslo\nJirka=pesko\nJan=pan\n".getBytes("UTF-8"));
        os.close();
        super.setUp();
    }

    @Override
    public void tearDown() throws Exception {
        deleteRec(dir);
    }

    static void deleteRec(File dir) throws IOException {
        if (dir == null) {
            return;
        }
        File[] arr = dir.listFiles();
        if (arr != null) {
            for (File f : arr) {
                deleteRec(f);
            }
        }
        dir.delete();
    }
    @Test public void testListGames() throws Exception {
        File usersDir = new File(dir, "users");
        usersDir.mkdirs();
        File fJarda = new File(usersDir, "Jarda");
        {
            Properties p = new Properties();
            p.setProperty("email", "jar@da.cz");
            p.setProperty("permission.games", "true");
            p.store(new FileOutputStream(fJarda), "");
        }
        File fJirka = new File(usersDir, "Jirka");
        {
            Properties p = new Properties();
            p.setProperty("email", "jir@ka.cz");
            p.store(new FileOutputStream(fJirka), "");
        }
        File fJan = new File(usersDir, "Jan");
        {
            Properties p = new Properties();
            p.setProperty("email", "j@an.cz");
            p.store(new FileOutputStream(fJan), "");
        }

        String logRoot = webResource.path("login").
            queryParam("name", "Jarda").
            queryParam("password", "heslo").
            accept(MediaType.TEXT_PLAIN).
            put(String.class);
        String logJirka = webResource.path("login").
            queryParam("name", "Jirka").
            queryParam("password", "pesko").
            accept(MediaType.TEXT_PLAIN).
            put(String.class);
        String logJan = webResource.path("login").
            queryParam("name", "Jan").
            queryParam("password", "pan").
            accept(MediaType.TEXT_PLAIN).
            put(String.class);

        GameId s = webResource.path("games").queryParam("white", "Jan")
                .queryParam("loginID", logJan)
                .queryParam("black", "Jirka").post(GameId.class);
            webResource.path("games/" + s.getId())
                .queryParam("loginID", logJan)
                .queryParam("player", "Jan").queryParam("move", "RESIGN").put(GameId.class);

        GenericType<List<GameId>> gType = new GenericType<List<GameId>>() {};
        List<GameId> all = webResource.path("games/").queryParam("loginID", logRoot).accept(MediaType.TEXT_XML).get(gType);
        boolean found = false;
        for (GameId id : all) {
            if (id.getId().equals(s.getId())) {
                found = true;
                break;
            }
        }
        assertTrue("List of games shall contai all games: " + all, found);

        Game end = webResource.path("games/" + s.getId()).queryParam("loginID", logRoot).accept(MediaType.TEXT_XML).get(Game.class);
        assertEquals("One can see status of games with priviledges", GameStatus.blackWon, end.getId().getStatus());
    }
}
